from database_mysql_local.generic_crud import GenericCRUD
from logger_local.MetaLogger import MetaLogger

from .StorageConstants import LOGGER_CODE_OBJECT


class StorageDB(GenericCRUD, metaclass=MetaLogger, object=LOGGER_CODE_OBJECT):
    def __init__(self) -> None:
        super().__init__(default_schema_name='storage', default_table_name='storage_table',
                         default_view_table_name='storage_view', default_column_name='storage_id')

    def upload_to_database(self, *, file_path: str, filename: str, region: str, storage_type_id: int, file_type_id: int,
                           extension_id: int, url: str = None) -> int:
        data_dict = {"path": file_path, "filename": filename, "region": region, "url": url,
                     "storage_type_id": storage_type_id, "file_type_id": file_type_id,
                     "file_extension_id": extension_id}
        storage_id = self.insert(data_dict=data_dict)
        return storage_id

    def delete(self, *, remote_path: str, filename: str, region: str) -> int:
        where = "path = %s AND filename = %s AND region = %s"
        params = (remote_path, filename, region)
        storage_id = super().select_one_value_by_where(where=where, params=params, select_clause_value="storage_id")
        super().delete_by_where(where=where, params=params)
        return storage_id



# Abstract Base Classes (abc)
from abc import ABC, abstractmethod
from logger_local.MetaLogger import ABCMetaLogger


# Interface for all storage services


class StorageInterface(ABC, metaclass=ABCMetaLogger):
    """
    Method uploads a file from local_path to the cloud in remoth_path, and names the file filname
    """

    @abstractmethod
    def upload_file(self, *, local_file_path: str, remote_path: str, url: str = None) -> int or None:
        pass

    @abstractmethod
    def download_file(self, *, remote_path: str, local_path: str) -> None:
        pass

    @abstractmethod
    def delete_by_remote_path_filename(self, remote_path: str, filename: str) -> None:
        pass

    @abstractmethod
    def delete_by_storage_id(self, storage_id: str, filename: str) -> None:
        pass




import setuptools

PACKAGE_NAME = "storage-local"
PACKAGE_NAME_UNDERSCORE = PACKAGE_NAME.replace("-", "_")
package_dirs = ["aws_s3_storage_local"]

setuptools.setup(
    name=PACKAGE_NAME,
    version='0.1.48',  # https://pypi.org/project/storage-local/
    author="Circles",
    author_email="info@circlez.ai",
    description="PyPI Package for Circles Storage functions",
    long_description="This is a package for sharing common S3 functions used in different repositories",
    long_description_content_type="text/markdown",
    url=f"https://github.com/circles-zone/{PACKAGE_NAME}-python-package",
    packages=[f"{PACKAGE_NAME_UNDERSCORE}/{package_dir}" for package_dir in package_dirs],
    package_dir={f"{PACKAGE_NAME_UNDERSCORE}/{package_dir}": f'{package_dir}/src' for package_dir in package_dirs},
    package_data={f"{PACKAGE_NAME_UNDERSCORE}/{package_dir}": ['*.py'] for package_dir in package_dirs},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    install_requires=["requests",
                      "boto3>=1.28.44",
                      "database-mysql-local>=0.0.107",
                      "logger-local>=0.0.135",
                      "user-context-remote>=0.0.75",
                      "python-sdk-remote>=0.0.93"]
)



from enum import Enum
from logger_local.LoggerComponentEnum import LoggerComponentEnum
from python_sdk_remote.utilities import our_get_env


# File-Type according to storage.file_type_ml_table
# TODO Sql2Code
class FileTypeEnum(Enum):
    PROFILE_IMAGE = 1
    COVERAGE_IMAGE = 2
    PERSONAL_INTODUCTION_VIDEO = 3
    SCANNED_DRIVING_LICENSE = 4
    PDF = 5
    SCANNED_PASSPORT = 6
    GOOGLE_CONTACTS_CSV = 7
    LINKEDIN_CONTACTS_CSV = 8
    OUTLOOK_CONTACTS_CSV = 9
    ORGANIZATION_PROFILE_LOGO = 10
    PROFILE_HTML = 11


USER_LOCAL_PYTHON_PACKAGE_COMPONENT_ID = 207  # 13?
USER_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME = "storage-main-local-python-package"

LOGGER_CODE_OBJECT = {
    'component_id': USER_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': USER_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'tal.g@circ.zone'
}

LOGGER_TEST_OBJECT = {
    'component_id': USER_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': USER_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Unit_Test.value,
    'testing_framework': LoggerComponentEnum.testingFramework.pytest.value,
    'developer_email': 'tal.g@circ.zone'
}

# TODO Should we change this to TEST_STORAGE_FILE_TYPE_ID?
STORAGE_TYPE_ID = 1
FILE_TYPE_ID = 1

# TODO Add all values from storage.file_extension_table


# todo: use sql2code
class ExtensionsEnum(Enum):
    txt = 1
    pdf = 2
    doc = 3
    xls = 4
    jpg = 5


AWS_DEFAULT_REGION = our_get_env("AWS_DEFAULT_REGION")
AWS_DEFAULT_STORAGE_BUCKET_NAME = our_get_env("AWS_DEFAULT_STORAGE_BUCKET_NAME")


import os

import boto3
from logger_local.MetaLogger import MetaLogger
from python_sdk_remote.utilities import our_get_env

from .StorageConstants import STORAGE_TYPE_ID, FILE_TYPE_ID, LOGGER_CODE_OBJECT
from .StorageDB import StorageDB
from .StorageInterface import StorageInterface
from .StorageConstants import ExtensionsEnum


AWS_ACCESS_KEY_ID = our_get_env("AWS_ACCESS_KEY_ID")
AWS_SECRET_ACCESS_KEY = our_get_env("AWS_SECRET_ACCESS_KEY")


class AwsS3Storage(StorageInterface, metaclass=MetaLogger, object=LOGGER_CODE_OBJECT):

    def __init__(self, bucket_name: str, region: str) -> None:
        self.region = region
        self.bucket_name = bucket_name
        self.storage_database = StorageDB()
        self.boto3_client = boto3.client('s3',
                                   aws_access_key_id=AWS_ACCESS_KEY_ID,
                                   aws_secret_access_key=AWS_SECRET_ACCESS_KEY)

    @staticmethod
    # TODO Move this method out of this class
    def get_filename_from_path(path: str) -> str:
        return os.path.basename(path)

    def upload_file(self, *, local_file_path: str, remote_path: str, url: str = None) -> int or None:
        """uploads file to S3"""
        read_binary = 'rb'
        filename = self.get_filename_from_path(local_file_path)
        # determine type of file
        ext=os.path.splitext(local_file_path)[1]
        with open(local_file_path, read_binary) as file_obj:
            file_contents = file_obj.read()

        # Upload the file to S3 with the CRC32 checksum
        response = self.boto3_client.put_object(
            Bucket=self.bucket_name,
            Key=remote_path + filename,
            Body=file_contents,
            ChecksumAlgorithm='crc32'
        )


        if 'ETag' in response:
            # TODO: constracts should be replaced with parameters
            storage_id = self.storage_database.upload_to_database(file_path=remote_path, filename=filename,
                                                                  region=self.region, storage_type_id=STORAGE_TYPE_ID,
                                                                  file_type_id=FILE_TYPE_ID, extension_id=(self.get_eum_by_extension(ext)),
                                                                  url=url)
            return storage_id
        return None

    # download a file from s3 to local_file_path
    def download_file(self, remote_path: str, local_file_path: str) -> None:
        self.boto3_client.download_file(self.bucket_name, remote_path, local_file_path)

    # logical delete

    def delete_by_remote_path_filename(self, remote_path: str, filename: str) -> None:
        self.storage_database.delete(remote_path=remote_path, filename=filename, region=self.region)
    
    def delete_by_storage_id(self, storage_id: str, filename: str) -> None:
        self.storage_database.delete(remote_path=storage_id, filename=filename, region=self.region)

    def get_eum_by_extension(file_extension) -> int:
        if file_extension == '.txt':
            return ExtensionsEnum.txt.value
        if file_extension == '.doc':
            return ExtensionsEnum.doc.value
        if file_extension == '.pdf':
            return ExtensionsEnum.pdf.value
        if file_extension == '.xls':
            return ExtensionsEnum.xls.value
        if file_extension == '.jpg':
            return ExtensionsEnum.jpg.value
        else:
            raise ValueError(f"Unsupported file extension: {file_extension}")





import os
import re
import shutil

import requests
from database_mysql_local.generic_crud import GenericCRUD
from logger_local.MetaLogger import MetaLogger

from .AWSStorage import AwsS3Storage
from .StorageConstants import LOGGER_CODE_OBJECT, AWS_DEFAULT_STORAGE_BUCKET_NAME, AWS_DEFAULT_REGION


class Storage(GenericCRUD, metaclass=MetaLogger, object=LOGGER_CODE_OBJECT):

    def __init__(self, is_test_data: bool = False):
        super().__init__(default_schema_name='storage', default_table_name='storage_table',
                         default_view_table_name='storage_view', default_column_name='storage_id')
        # TODO: get_s3 per region & bucket, according to user location.
        # TODO aws_bucket_name=get_aws_bucket_name( user_context )
        # TODO aws_region = get_aws_region( user_context )
        self.s3 = AwsS3Storage(bucket_name=AWS_DEFAULT_STORAGE_BUCKET_NAME, region=AWS_DEFAULT_REGION)
        self.is_test_data = is_test_data

    # returns the folder name from DB according to file_type_id
    def _get_file_type_by_file_type_id(self, file_type_id: int) -> str:
        file_type = self.select_one_value_by_column_and_value(view_table_name="file_type_view",
                                                              column_name="file_type_id",
                                                              column_value=file_type_id,
                                                              select_clause_value="file_type")
        return file_type

    def _get_region_and_folder_by_file_type_id(self, file_type_id: int) -> [str, str]:
        folder = self._get_file_type_by_file_type_id(file_type_id)
        # TODO Replace with a new method in user-context: region = user-context.get_effective_region()
        return [folder, AWS_DEFAULT_REGION]

    #
    def put(self, file_type_id: int, local_file_path: str) -> int:
        """uploads a file from the local computer to S3 and returns the storage_id of the file in the storage DB

        Args:
            profile_id int: user ID
            file_type_id int: type of the file - 
                                1 - Profile Image
                                2 - Coverage Image
                                3 - Personal Introduction Video
                                4 - Scanned Diving License 
                                5 - Scanned Passport
            filename string: file name including extension, i.e.test.txt
            local_file_path string: path to the file location, i.e. path/to/file/
        Returns:
            int: ID of the file in the storage DB
        """
        folder_and_region = self._get_region_and_folder_by_file_type_id(file_type_id)
        storage_id = self.s3.upload_file(local_file_path=local_file_path,
                                         remote_path=folder_and_region[0] + '/')
        return storage_id

    @staticmethod
    def preserve_letters_from_string(input_string: str) -> str:
        """Preserves only letters and spaces in a given string

        Args:
            input_string string: unmodified string
        """
        # Use a regular expression to match only letters (A-Z and a-z)
        pattern = r"[^a-zA-Z\s]"
        preserved_string = re.sub(pattern, "", input_string)
        return preserved_string

    def download_by_storage_id(self, storage_id: int, target_local_file_path: str) -> None:
        """Downlaods file from S3 to the local computer using only storage id

        Args:
            storage_id int: Row number to get information from storage_table
        """
        result = self.select_one_tuple_by_column_and_value(
            select_clause_value="created_user_id, path, filename", column_value=storage_id)
        if not result:
            raise ValueError(f"Storage ID {storage_id} not found in the database")
        profile_id, folder, filename = result
        self.s3.download_file(folder + filename, target_local_file_path)

    def download(self, *, file_type_id: int, remote_filename: str, local_path: str) -> None:
        """Downlaods file from S3 to local computer

        Args:
            file_type_id int: 1 - Profile Image
                                2 - Coverage Image
                                3 - Personal Introduction Video
                                4 - Scanned Diving License 
                                5 - Scanned Passport
            filename string: file name include extension, i.e.test.txt
            local_path string: where to save the file, include file extension,
            i.e.path/to/file/downloaded_test.txt
        """
        folder_and_region = self._get_region_and_folder_by_file_type_id(file_type_id)
        remote_file_path = folder_and_region[0] + '/' + remote_filename
        self.s3.download_file(remote_file_path, local_path)

    def save_image_in_storage_by_url(self, image_url: str, local_file_path: str, profile_id: int, file_type_id: int) -> None:
        self.logger.start(object={'image_url': image_url, 'local_file_path': local_file_path, 'profile_id': profile_id,
                                  'file_type_id': file_type_id})
        folder_and_region = self._get_region_and_folder_by_file_type_id(file_type_id)
        remote_file_path = folder_and_region[0] + '/'
        response = requests.get(image_url, stream=True)

        # Check if the image was retrieved successfully
        if response.status_code == requests.codes.ok:
            # Set decode_content value to True, otherwise the downloaded image file's size will be zero.
            response.raw.decode_content = True

            # Open a local file with wb ( write binary ) permission.
            with open(local_file_path, 'wb') as f:
                shutil.copyfileobj(response.raw, f)

            self.s3.upload_file(local_file_path=local_file_path, remote_path=remote_file_path,
                                url=image_url)
            os.remove(local_file_path)
            self.logger.end('Image successfully downloaded to ' + local_file_path)
        else:
            self.logger.error('Image couldn\'t be retrieved')

    '''
    # Not tested yet
    # TODO: later we may want to save also the html content of the url in the storage
    def download_html(self, url, local_file_path: str) -> requests.models.Response:
        METHOD_NAME = 'download_html'
        self.logger.start(METHOD_NAME, object={'url': url, 'local_file_path': local_file_path})
        response = requests.get(url)
        response.raise_for_status()
        with open(local_file_path, 'w', encoding='utf-8') as file:
            file.write(response.text)
        self.logger.end(METHOD_NAME, object={'response': response})
        return response

    # TODO: maybe we shall name it save_html_in_storage_by_url
    def save_url_in_storage(self, url: str, file_type_id: int, local_file_path: str) -> int:
        METHOD_NAME = 'save_url_in_storage'
        self.logger.start(METHOD_NAME, object={'url': url, 'file_type_id': file_type_id, 'local_file_path': local_file_path})
        folder_and_region = self._get_region_and_folder_by_file_type_id(file_type_id)
        remote_file_path = folder_and_region[0] + '/'
        response = self.download_html(url, local_file_path)
        if response.status_code == requests.codes.ok:
            self.logger.info(log_message=METHOD_NAME + ': ' + 'File downloaded successfully',
                             object={'url': url, 'local_file_path': local_file_path})
            storage_id = self.s3.upload_file(local_file_path=local_file_path, remote_path=remote_file_path,
                                             url=url)
        else:
            self.logger.warning(log_message=METHOD_NAME + ': ' + 'File could not be downloaded',
                                object={'url': url, 'local_file_path': local_file_path})
            storage_id = None
        self.logger.end(METHOD_NAME, object={'storage_id': storage_id})
        return storage_id
    '''

    # TODO: later we may want to save also the html content of the url in the storage
    def save_url_in_storage(self, url: str, file_name: str) -> int:
        METHOD_NAME = 'save_url_in_storage'
        self.logger.start(METHOD_NAME, object={'url': url, 'file_name': file_name})
        script_directory = os.path.dirname(os.path.abspath(__file__))
        local_file_path = os.path.join(script_directory, file_name)
        folder_and_region = ["Url", AWS_DEFAULT_REGION]
        remote_file_path = folder_and_region[0] + '/'
        # Create a txt file with the url as it's content
        with open(local_file_path, 'w', encoding='utf-8') as file:
            file.write(url)
        storage_id = self.s3.upload_file(local_file_path=local_file_path, remote_path=remote_file_path,
                                         url=url)
        # Delete local file
        os.remove(local_file_path)
        self.logger.end(METHOD_NAME, object={'storage_id': storage_id})
        return storage_id
